#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
# Univention Management Console module:
#   Maildisclaimer
#
# Copyright 2017 bitbone AG
#
# http://www.bitbone.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

__package__='' 	# workaround for PEP 366
import ldap
import listener
import univention.debug as ud
import os
import grp
import re

name = "maildisclaimer"
description = "Maildisclaimer template sync"
filter = "(|(objectClass=maildisclaimerTemplate)(objectClass=maildisclaimerFile))"
attribute = []
modrdn = "1"

maildisclaimer_opts = {
    "template_path": "/etc/maildisclaimer/templates",
    "image_path": "/etc/maildisclaimer/images",
    "uid": "maildisclaimer",
    "gid": "maildisclaimer",
}

TYPES = ['txt', 'html']

def _get_safe_filename(filename):
    return re.sub(r"(\/|\\|\0)", "", filename)

def _set_permission(filename):
    os.chown(filename, maildisclaimer_opts.uid, maildisclaimer_opts.gid)

def add(template):
    if template["name"] == "global":
        return

    for t in TYPES:
        filename = "%s/%s.%s" % (maildisclaimer_opts["template_path"], template["name"], t)
        ud.debug(ud.LISTENER, ud.INFO, "create template %s" % (filename))
        f = open(filename, "w")
        f.write(template[t])
        f.close()

        link = "%s/global.%s" % (maildisclaimer_opts["template_path"], t)
        if template["global"] == "TRUE":
            if os.path.islink(link):
                os.unlink(link)
            os.symlink(filename, link)
        elif template["global"] == "FALSE":
            # only remove symlink if it points to the current template
            if os.path.realpath(link) == filename:
                os.unlink(link)
        else:
            ud.debug(ud.LISTENER, ud.INFO, "this should not happen")

def remove(filename):
    for t in TYPES:
        path = "%s/%s.%s" % (
                maildisclaimer_opts["template_path"],
                _get_safe_filename(filename), t)
        ud.debug(ud.LISTENER, ud.INFO, "removing template %s" % (filename))

        if os.path.isfile(path):
            os.remove(path)

        path = "%s/global.%s" % (
                maildisclaimer_opts["template_path"], t)

        # if the global link is not valid anymore, remove it
        if not os.path.exists(path) and os.path.islink(path):
            os.remove(path)

def add_image(image):
    filename = "%s/%s" % (
            maildisclaimer_opts["image_path"],
            _get_safe_filename(image["name"]))
    ud.debug(ud.LISTENER, ud.INFO, "create file %s" % (filename))
    f = open(filename, "w")
    f.write(image["content"])
    f.close()

def remove_image(filename):
    path = "%s/%s" % (
            maildisclaimer_opts["image_path"],
            _get_safe_filename(filename))
    if os.path.isfile(path):
        os.remove(path)

def create_template_object(obj):
    key_map = { "templateHTML": "html", "templateText": "txt", "cn": "name", "isGlobal": "global" }
    template = { "templateHTML": "", "templateText": "", "cn": "", "isGlobal": "FALSE" }
    output = {}

    for key in template:
        if obj.has_key(key) and len(obj[key]) >= 1:
            output[key_map[key]] = obj[key][0]
        else:
            output[key_map[key]] = template[key]

    ud.debug(ud.LISTENER, ud.INFO, "create template object: %s" % (repr(output)))

    return output

def create_image_object(obj):
    key_map = { "cn": "name", "fileContent": "content" }
    template = { "cn": "", "fileContent": "missing" }
    output = {}

    for key in template:
        if obj.has_key(key) and len(obj[key]) >= 1:
            output[key_map[key]] = obj[key][0]
        else:
            output[key_map[key]] = template[key]

    ud.debug(ud.LISTENER, ud.INFO, "create image object: %s" % (repr(output)))

    return output

def initialize(): pass
def clean(): pass
def prerun(): pass
def postrun(): pass
def setdata(key, value): pass

def handler(dn, new, old, command):
    ud.debug(ud.LISTENER, ud.INFO, "running maildisclaimer handler")

    if new and not old:
        # add
        if "maildisclaimer/images" in new['univentionObjectType']:
            ud.debug(ud.LISTENER, ud.INFO, "add image")
            add_image(create_image_object(new))
        elif "maildisclaimer/templates" in new['univentionObjectType']:
            ud.debug(ud.LISTENER, ud.INFO, 'add template')
            add(create_template_object(new))
    elif old and not new:
        # delete
        if "maildisclaimer/images" in old['univentionObjectType']:
            ud.debug(ud.LISTENER, ud.INFO, "remove image")
            remove_image(old["cn"][0])
        elif "maildisclaimer/templates" in old['univentionObjectType']:
            ud.debug(ud.LISTENER, ud.INFO, 'remove template')
            remove(old["cn"][0])
    elif old and new:
        # modify
        if "maildisclaimer/images" in new['univentionObjectType']:
            ud.debug(ud.LISTENER, ud.INFO, "modify image")
            add_image(create_image_object(new))
        elif "maildisclaimer/templates" in new['univentionObjectType']:
            ud.debug(ud.LISTENER, ud.INFO, 'modify template')
            ud.debug(ud.LISTENER, ud.INFO, repr(old))
            ud.debug(ud.LISTENER, ud.INFO, repr(new))
            add(create_template_object(new))
